# Nico Schottelius, 2020-12-05, GPLv3+
# Developed at Hack4Glarus 2020 Winter - https://hack4glarus.ch

from html.parser import HTMLParser
import urllib.request
import re

from django.core.management.base import BaseCommand
from django.contrib.auth import get_user_model
from ipv6ula.models import ULA

url = "https://www.sixxs.net/tools/grh/ula/list/"


class MyHTMLParser(HTMLParser):
    def __init__(self):
        self.tbody_count = 0
        self.in_table = False
        self.in_row = False

        self.col_index = 0

        super().__init__()

        self.results = []

        self.th = False
        self.prefix = False
        self.name = False
        self.org = False
        self.website = False

    def handle_starttag(self, tag, attrs):
        if tag == "th":
            self.th = True

        if self.in_table and tag == "tr":
            self.in_row = True
            self.col_index = 0

        elif self.in_row and tag == "td":
            self.col_index += 1

            if self.col_index == 2:
                self.name = True
            elif self.col_index == 3:
                self.org = True

        elif self.col_index == 1 and tag == "a":
            self.prefix = True
        elif self.col_index == 4 and tag == "a":
            self.website = True

    def handle_endtag(self, tag):
        if tag == "th":
            self.th = False
        elif tag == "tr":
            self.in_row = False


    def handle_data(self, data):
        if self.th and data == "Prefix":
            self.in_table = True

        if self.prefix:
            self.record = {}
            self.record['prefix'] = data

            self.results.append(self.record)

            self.prefix = False

        elif self.name:
            self.record['name'] = data
            self.name = False
        elif self.org:
            if data == '\xa0':
                data = ''
            self.record['organization'] = data
            self.org = False
        elif self.website:
            self.record['website'] = data
            self.website = False

    def report(self):
        for record in self.results:
            print(record)

class Command(BaseCommand):
    help = "Import prefixes registered on sixxs"

    def handle(self, *args, **options):
        default_owner = get_user_model().objects.get(username='admin')

        parser = MyHTMLParser()
        response = urllib.request.urlopen(url)
        html = "\n".join([ line.decode('utf-8') for line in response.readlines() ])

        parser.feed(html)

        for record in parser.results:
            db_prefix=re.search("(.*)/", record['prefix']).groups()[0] # strip away /48

            record['owner'] = default_owner
            del record['prefix']

            ULA.objects.get_or_create(prefix=db_prefix, defaults = record)


if __name__ == '__main__':
    parser = MyHTMLParser()
    response = urllib.request.urlopen(url)
    html = "\n".join([ line.decode('utf-8') for line in response.readlines() ])

    parser.feed(html)
    parser.report()
