from django.views.generic.list import ListView
from django.views.generic.edit import CreateView
from django.views.generic.base import TemplateView

from django.contrib.auth import views as auth_views
from django.contrib.auth.mixins import LoginRequiredMixin
from django.contrib.messages.views import SuccessMessageMixin

from django.contrib.auth import logout
from django.shortcuts import redirect

import ipaddress

from django.conf import settings

from .models import *
from .forms import *
from .utils import *

class IndexView(ListView):
    model = ULA
    queryset = ULA.objects.order_by('prefix')

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['count'] = self.get_queryset().count()
        return context

class RandowULAView(TemplateView):
    model = ULA
    template_name = "ipv6ula/random_ula.html"

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['prefix'] = str(random_ula_net())
        return context

class GenerateSubmitView(LoginRequiredMixin, SuccessMessageMixin, CreateView):
    model = ULA

    login_url = '/login/'
    success_url = '/'
    success_message = "%(the_prefix)s/48 was created successfully"

    gen_method = "undef"

    def get_success_message(self, cleaned_data):
        return self.success_message % dict(cleaned_data,
                                           the_prefix = self.object.prefix)

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['method'] = self.gen_method
        return context

class GenerateView(GenerateSubmitView):
    form_class = ULAGenerateForm
    gen_method = "generate"

    def form_valid(self, form):
        form.instance.owner = self.request.user
        form.instance.prefix = str(random_ula_net())
        return super(GenerateView, self).form_valid(form)

class SubmitView(GenerateSubmitView):
    form_class = ULAForm
    gen_method = "submit"

    def form_valid(self, form):
        form.instance.owner = self.request.user
        return super().form_valid(form)

class LoginView(auth_views.LoginView):
    template_name = 'ipv6ula/login.html'

def logout_view(request):
    logout(request)
    return redirect("/")
